# -*- coding: utf-8 -*-

"""
***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************
"""
from PyQt5.QtGui import QIcon
from qgis.core import QgsProcessingProvider
from processing.core.ProcessingConfig import Setting, ProcessingConfig

from .CalculateCorridor import CalculateCorridor
from .CreateLinkset import CreateLinkset
from .CreateGraph import CreateGraph
from .CreateProject import CreateProject
from .CalculateLocalMetric import CalculateLocalMetric
from .CalculateGlobalMetric import CalculateGlobalMetric


class GraphabProvider(QgsProcessingProvider):


    def __init__(self, Graphab):
        super().__init__()
        self.Graphab = Graphab

    def load(self):
        """In this method we add settings needed to configure our
        provider.
        """
        ProcessingConfig.settingIcons[self.name()] = self.icon()
        # Deactivate provider by default
        ProcessingConfig.addSetting(Setting(self.name(), 'ACTIVATE_GRAPHAB',
                                            'Activate', True))

        ProcessingConfig.readSettings()
        self.refreshAlgorithms()
        return True

    def unload(self):
        """Setting should be removed here, so they do not appear anymore
        when the plugin is unloaded.
        """
        ProcessingConfig.removeSetting('ACTIVATE_GRAPHAB')

    def isActive(self):
        """Return True if the provider is activated and ready to run algorithms"""
        return ProcessingConfig.getSetting('ACTIVATE_GRAPHAB')

    def setActive(self, active):
        ProcessingConfig.setSettingValue('ACTIVATE_GRAPHAB', active)

    def id(self):
        """This is the name that will appear on the toolbox group.

        It is also used to create the command line name of all the
        algorithms from this provider.
        """
        return 'graphab'

    def name(self):
        """This is the localised full name.
        """
        return 'Graphab'

    def icon(self):
        """We return the default icon.
        """
        return QIcon(':/plugins/graphab-for-qgis/icons/icon.png')

    def loadAlgorithms(self):
        """Here we fill the list of algorithms in self.algs.

        This method is called whenever the list of algorithms should
        be updated. If the list of algorithms can change (for instance,
        if it contains algorithms from user-defined scripts and a new
        script might have been added), you should create the list again
        here.

        In this case, since the list is always the same, we assign from
        the pre-made list. This assignment has to be done in this method
        even if the list does not change, since the self.algs list is
        cleared before calling this method.
        """
        for alg in [CreateProject(self.Graphab), CreateLinkset(self.Graphab), CreateGraph(self.Graphab),
                    CalculateGlobalMetric(self.Graphab), CalculateLocalMetric(self.Graphab),
                    CalculateCorridor(self.Graphab)]:
            self.addAlgorithm(alg)
